#pragma once

#include "Vector.h"

#include <vector>
#include <complex>

template<typename T, std::size_t M = 3, std::size_t N = 3>
class Matrix
{
public:
    enum class InputType
    {
        UNIFORM,
        DIAGONAL
    };

    Matrix(T value = 0.0);

    Matrix(T value, InputType inputType);

    void applyTo(Vector<T, N>& v) const;

    std::size_t getFlatIndex(std::size_t rowId, std::size_t columnId) const;

    T getItem(std::size_t rowId, std::size_t columnId) const;

    bool isIdentity() const;

    bool isSquare() const;

    bool isEqual(const Matrix<T, M, N>& matrix) const;

    void setDiagonals(T value);

    void setDiagonals(const std::vector<T>& values);

    void setItem(std::size_t rowId, std::size_t columnId, T value);

    bool operator==(const Matrix<T, M, N>& rhs) const
    {
       return isEqual(rhs);
    }

    bool operator!=(const Matrix<T, M, N>& rhs) const
    {
      return !operator==(rhs);
    }

private:
    std::vector<T> mData;
};

using Matrix3x3 = Matrix<double, 3, 3>;
using SquareMatrix3 = Matrix<double, 3, 3>;

using Matrix2x2 = Matrix<double, 2, 2>;
using SquareMatrix2 = Matrix<double, 2, 2>;

using SquareMatrix4 = Matrix<double, 4, 4>;

using ComplexMatrix3x3 = Matrix<std::complex<double>, 3, 3>;
using ComplexMatrix2x2 = Matrix<std::complex<double>, 3, 3>;