#include "Vector.h"

#include <cmath>
#include <algorithm>
#include <numeric>
#include <stdexcept>
#include <functional>

template<typename T, std::size_t DIM>
Vector<T, DIM>::Vector(Primitive p)
{
    mData = std::vector<T>(DIM, 0.0);
    switch(p)
    {
    case Primitive::UNIT_X:
        mData[0] = 1.0;
        break;
    case Primitive::UNIT_Y:
        mData[1] = 1.0;
        break;
    case Primitive::UNIT_Z:
        mData[2] = 1.0;
        break;
    case Primitive::NEGATIVE_X:
        mData[0] = -1.0;
        break;
    case Primitive::NEGATIVE_Y:
        mData[1] = -1.0;
        break;
    case Primitive::NEGATIVE_Z:
        mData[2] = -1.0;
        break;
    }
}

template<typename T, std::size_t DIM>
Vector<T, DIM>::Vector(const std::vector<T>& values)
{
    mData = std::vector<T>(DIM, 0.0);

    if (values.empty())
    {
        return;
    }

    for (std::size_t idx = 0; idx < std::min(values.size(), DIM); idx++)
    {
        mData[idx] = values[idx];
    }
}

template<typename T, std::size_t DIM>
Vector<T, DIM>::~Vector()
{
};

template<typename T, std::size_t DIM>
Vector<T, DIM> Vector<T, DIM>::add(const Vector<T, DIM>& v) const
{
    Vector<T, DIM> result = *this;
    result.inPlaceAdd(v);
    return result;
}

template<typename T, std::size_t DIM>
void Vector<T, DIM>::inPlaceAdd(const Vector<T, DIM>& v)
{
    std::transform(mData.begin(), mData.end(), v.mData.begin(), mData.begin(), std::plus<T>());
}

template<typename T, std::size_t DIM>
bool Vector<T, DIM>::equals(const Vector<T, DIM>& v) const
{
    return mData == v.mData;
}

template<typename T, std::size_t DIM>
void Vector<T, DIM>::inPlaceMultiply(double v)
{
    //std::transform(mData.begin(), mData.end(), mData.begin(), std::bind(std::multiplies<double>(), std::placeholders::_1, v));
}

template<typename T, std::size_t DIM>
Vector<T, DIM> Vector<T, DIM>::multiply(double v) const
{
    Vector<T, DIM> result = *this;
    result.inPlaceMultiply(v);
    return result;
}

template<typename T, std::size_t DIM>
void Vector<T, DIM>::reverseDirection()
{
    inPlaceMultiply(-1.0);
}

template<typename T, std::size_t DIM>
T Vector<T, DIM>::getEntry(std::size_t idx) const
{
    if (idx >= DIM)
    {
        throw std::range_error("Out of bounds access for Vector");
    }
    return mData[idx];
}

template<typename T, std::size_t DIM>
double Vector<T, DIM>::getLength() const
{
    return std::sqrt(getSelfInnerProduct());
}

template<typename T, std::size_t DIM>
double Vector<T, DIM>::getSelfInnerProduct() const
{
    return 0.0;
    //return std::inner_product(mData.begin(), mData.end(), mData.begin(), 0.0);
}

template<typename T, std::size_t DIM>
double Vector<T, DIM>::innerPoduct(const Vector<T, DIM>& v) const
{
    return 0.0;
    //return std::inner_product(mData.begin(), mData.end(), v.mData.begin(), 0.0);
}

template<typename T, std::size_t DIM>
Vector<T, DIM> Vector<T, DIM>::crossProduct(const Vector<T, DIM>& v) const
{
    return Vector<T, DIM>();
    //return Vector(v.mY * mZ - v.mZ * mY, v.mZ * mX - v.mX * mZ, v.mX * mY - v.mY * mX);
}

template<typename T, std::size_t DIM>
Vector<T, DIM> Vector<T, DIM>::getNormalized() const
{
    const auto length = getLength();
    if (length == 0.0)
    {
        throw std::logic_error("Divide by zero getting normalizing vector");
    }
    return (*this).multiply(1.0 / length);
}

template<typename T, std::size_t DIM>
void Vector<T, DIM>::scale(const std::vector<double>& factors)
{
    //std::transform(mData.begin(), mData.end(), factors.begin(), mData.begin(), std::multiplies<double>());
}

template class Vector<double, 2>;
template class Vector<double, 3>;
template class Vector<double, 4>;

template class Vector<std::complex<double>, 2>;
template class Vector<std::complex<double>, 3>;
