#pragma once

#include <vector>
#include <complex>

template<typename T, std::size_t DIM = 3>
class Vector
{
public:
    enum class Primitive
    {
        UNIT_X,
        UNIT_Y,
        UNIT_Z,
        NEGATIVE_X,
        NEGATIVE_Y,
        NEGATIVE_Z
    };

    Vector(Primitive p);

    Vector(const std::vector<T>& values = {});

    ~Vector();

    Vector<T, DIM> add(const Vector<T, DIM>& v) const;

    void inPlaceAdd(const Vector<T, DIM>& v);

    double innerPoduct(const Vector<T, DIM>& v) const;

    Vector<T, DIM> crossProduct(const Vector<T, DIM>& v) const;

    bool equals(const Vector<T, DIM>& v) const;

    void reverseDirection();

    T getEntry(std::size_t idx) const;

    double getLength() const;

    Vector<T, DIM> getNormalized() const;

    double getSelfInnerProduct() const;

    Vector<T, DIM> multiply(double s) const;

    void inPlaceMultiply(double s);

    void scale(const std::vector<double>& factors);

    Vector<T, DIM> operator+(const Vector<T, DIM>& v) const
    {
        return add(v);
    }

    Vector<T, DIM> operator*(double s) const
    {
        return multiply(s);
    }

    void operator*=(double s)
    {
        inPlaceMultiply(s);
    }

    void operator+=(const Vector<T, DIM>& v)
    {
        add(v);
    }

    bool operator==(const Vector<T, DIM>& rhs) const
    {
        return equals(rhs);
    }

    bool operator!=(const Vector<T, DIM>& rhs) const
    {
        return !operator==(rhs);
    }

    T operator[](std::size_t idx) const
    {
        return getEntry(idx);
    }

private:
    std::vector<T> mData;
};

using Vector4 = Vector<double, 4>;
using Vector3 = Vector<double, 3>;
using Vector2 = Vector<double, 2>;

using ComplexVector3 = Vector<std::complex<double>, 3>;
using ComplexVector2 = Vector<std::complex<double>, 2>;

