#pragma once

#include "PathElement.h"

#include <vector>
#include <memory>
#include <string>

using PathElementPtr = std::unique_ptr<PathElement<2> >;

class GeometryPathFeature
{
public:
    void setLocation(Vector2 offset)
    {
        mOffset = offset;
    }

    void addElement(PathElementPtr element)
    {
        mElements.push_back(std::move(element));
    }

    const Vector2& getOffset() const
    {
        return mOffset;
    }

    const std::vector<PathElementPtr>& getElements() const
    {
        return mElements;
    }

    PathOffsetType getPathOffsetType() const
    {
        return mOffsetType;
    }

    void setPathOffsetType(PathOffsetType offsetType)
    {
        mOffsetType = offsetType;
    }

private:
    Vector2 mOffset;
    PathOffsetType mOffsetType{PathOffsetType::RELATIVE_TO};
    std::vector<PathElementPtr> mElements;
};
using GeometryPathFeaturePtr = std::unique_ptr<GeometryPathFeature>;

class GeometryPath : public AbstractGeometricItem
{
public:
    ~GeometryPath();

    void addFeature(GeometryPathFeaturePtr feature);

    Bounds getBounds() const override;

    Type getType() const override;

    const std::vector<GeometryPathFeaturePtr>& getFeatures() const;

private:
    std::vector<GeometryPathFeaturePtr> mFeatures;
};
