#include "PathNode.h"

#include "Path.h"
#include "SceneInfo.h"

#include "SceneModel.h"
#include "PostscriptReader.h"

PathNode::PathNode(const Transform& transform, const std::string& psPath)
	: GeometryNode(transform),
	    mPathString(psPath)
{

}

std::unique_ptr<PathNode> PathNode::Create(const Transform& transform, const std::string& psPath)
{
	return std::make_unique<PathNode>(transform, psPath);
}

GeometryNode::Type PathNode::getType() 
{
	return GeometryNode::Type::Path;
}

const std::string& PathNode::getPathString() const
{
	return mPathString;
}

void PathNode::setPathString(const std::string& psPath)
{
	mPathString = psPath;
}

void PathNode::createOrUpdateGeometry(SceneInfo* sceneInfo)
{
    auto path = std::make_unique<GeometryPath>();
    PostscriptReader reader;
    reader.read(path.get(), mPathString);

    if (!mBackgroundItem)
    {
        mBackgroundItem = std::make_unique<SceneModel>(std::move(path));
        mBackgroundItem->setName(mName + "_Model");     
    }
    else
    {
        mBackgroundItem->updateGeometry(std::move(path));
    }
}